@@ -16,8 +16,14 @@
 #   License along with this library; if not, write to the Free Software
 #   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-import os, re, xml.dom.minidom, locale
+import codecs
+import os
+import re
+import xml.dom.minidom
+import xml.parsers.expat
+import locale
 import matemenu
+from gi.repository import GLib
 from Mozo import util
 
 class Menu:
@@ -27,13 +33,12 @@ class Menu:
 	dom = None
 
 class MenuEditor:
-	#lists for undo/redo functionality
-	__undo = []
-	__redo = []
-
 	def __init__(self):
 		self.locale = locale.getdefaultlocale()[0]
 		self.__loadMenus()
+		self.__undo = []
+		self.__redo = []
+
 
 	def __loadMenus(self):
 		self.applications = Menu()
@@ -42,11 +47,11 @@ class MenuEditor:
 		self.applications.tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.applications.visible_tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.applications.path = os.path.join(util.getUserMenuPath(), self.applications.tree.get_menu_file())
-		if not os.path.isfile(self.applications.path):
-			self.applications.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.applications.tree))
-		else:
+		try:
 			self.applications.dom = xml.dom.minidom.parse(self.applications.path)
-		self.__remove_whilespace_nodes(self.applications.dom)
+		except (IOError, xml.parsers.expat.ExpatError):
+			self.applications.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.applications.tree))
+		util.removeWhitespaceNodes(self.applications.dom)
 
 		self.settings = Menu()
 		self.settings.tree = matemenu.lookup_tree('mate-settings.menu', matemenu.FLAGS_SHOW_EMPTY|matemenu.FLAGS_INCLUDE_EXCLUDED|matemenu.FLAGS_INCLUDE_NODISPLAY|matemenu.FLAGS_SHOW_ALL_SEPARATORS)
@@ -54,19 +59,18 @@ class MenuEditor:
 		self.settings.tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.settings.visible_tree.sort_key = matemenu.SORT_DISPLAY_NAME
 		self.settings.path = os.path.join(util.getUserMenuPath(), self.settings.tree.get_menu_file())
-		if not os.path.isfile(self.settings.path):
-			self.settings.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.settings.tree))
-		else:
+		try:
 			self.settings.dom = xml.dom.minidom.parse(self.settings.path)
-		self.__remove_whilespace_nodes(self.settings.dom)
+		except (IOError, xml.parsers.expat.ExpatError):
+			self.settings.dom = xml.dom.minidom.parseString(util.getUserMenuXml(self.settings.tree))
+		util.removeWhitespaceNodes(self.settings.dom)
 
 		self.save(True)
 
 	def save(self, from_loading=False):
 		for menu in ('applications', 'settings'):
-			fd = open(getattr(self, menu).path, 'w')
-			fd.write(re.sub("\n[\s]*([^\n<]*)\n[\s]*</", "\\1</", getattr(self, menu).dom.toprettyxml().replace('<?xml version="1.0" ?>\n', '')))
-			fd.close()
+			with codecs.open(getattr(self, menu).path, 'w', 'utf-8') as f:
+				f.write(getattr(self, menu).dom.toprettyxml())
 		if not from_loading:
 			self.__loadMenus()
 
@@ -94,11 +98,11 @@ class MenuEditor:
 			except OSError:
 				pass
 			#reload DOM for each menu
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
+			try:
 				menu.dom = xml.dom.minidom.parse(menu.path)
-			self.__remove_whilespace_nodes(menu.dom)
+			except (IOError, xml.parsers.expat.ExpatError):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+			util.removeWhitespaceNodes(menu.dom)
 		#reset undo/redo, no way to recover from this
 		self.__undo, self.__redo = [], []
 		self.save()
@@ -111,6 +115,31 @@ class MenuEditor:
 				self.revertItem(child)
 		self.revertMenu(menu)
 
+	def revertItem(self, item):
+		if not self.canRevert(item):
+			return
+		self.__addUndo([item,])
+		try:
+			os.remove(item.get_desktop_file_path())
+		except OSError:
+			pass
+		self.save()
+
+	def revertMenu(self, menu):
+		if not self.canRevert(menu):
+			return
+		#wtf happened here? oh well, just bail
+		if not menu.get_desktop_file_path():
+			return
+		self.__addUndo([menu,])
+		file_id = os.path.split(menu.get_desktop_file_path())[1]
+		path = os.path.join(util.getUserDirectoryPath(), file_id)
+		try:
+			os.remove(path)
+		except OSError:
+			pass
+		self.save()
+
 	def undo(self):
 		if len(self.__undo) == 0:
 			return
@@ -119,20 +148,30 @@ class MenuEditor:
 		for file_path in files:
 			new_path = file_path.rsplit('.', 1)[0]
 			redo_path = util.getUniqueRedoFile(new_path)
-			data = open(new_path).read()
-			open(redo_path, 'w').write(data)
-			data = open(file_path).read()
-			open(new_path, 'w').write(data)
+
+			f_file_path = codecs.open(new_path, 'r', 'utf-8')
+			f_new_path = codecs.open(new_path, 'rw', 'utf-8')
+			f_redo_path = codecs.open(redo_path, 'rw', 'utf-8')
+
+			data = f_new_path.read()
+			f_redo_path.write(data)
+			data = f_file_path.read()
+			f_new_path.write(data)
+
+			f_file_path.close()
+			f_new_path.close()
+			f_redo_path.close()
+
 			os.unlink(file_path)
 			redo.append(redo_path)
-		#reload DOM to make changes stick
+		# reload DOM to make changes stick
 		for name in ('applications', 'settings'):
 			menu = getattr(self, name)
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
+			try:
 				menu.dom = xml.dom.minidom.parse(menu.path)
-			self.__remove_whilespace_nodes(menu.dom)
+			except (IOError, xml.parsers.expat.ExpatError):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+			util.removeWhitespaceNodes(menu.dom)
 		self.__redo.append(redo)
 
 	def redo(self):
@@ -143,24 +182,30 @@ class MenuEditor:
 		for file_path in files:
 			new_path = file_path.rsplit('.', 1)[0]
 			undo_path = util.getUniqueUndoFile(new_path)
-			data = open(new_path).read()
-			open(undo_path, 'w').write(data)
-			data = open(file_path).read()
-			open(new_path, 'w').write(data)
+			f_file_path = codecs.open(new_path, 'r', 'utf-8')
+			f_new_path = codecs.open(new_path, 'rw', 'utf-8')
+			f_undo_path = codecs.open(undo_path, 'rw', 'utf-8')
+			data = f_new_path.read()
+			f_undo_path.write(data)
+			data = f_file_path.read()
+			f_new_path.write(data)
 			os.unlink(file_path)
 			undo.append(undo_path)
+			f_file_path.close()
+			f_new_path.close()
+			f_undo_path.close()
 		#reload DOM to make changes stick
 		for name in ('applications', 'settings'):
 			menu = getattr(self, name)
-			if not os.path.isfile(menu.path):
-				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
-			else:
+			try:
 				menu.dom = xml.dom.minidom.parse(menu.path)
-			self.__remove_whilespace_nodes(menu.dom)
+			except (IOError, xml.parsers.expat.ExpatError):
+				menu.dom = xml.dom.minidom.parseString(util.getUserMenuXml(menu.tree))
+			util.removeWhitespaceNodes(menu.dom)
 		self.__undo.append(undo)
 
 	def getMenus(self, parent=None):
-		if parent == None:
+		if parent is None:
 			yield self.applications.tree.root
 			yield self.settings.tree.root
 		else:
@@ -179,7 +224,7 @@ class MenuEditor:
 
 	def canRevert(self, item):
 		if item.get_type() == matemenu.TYPE_ENTRY:
-			if util.getItemPath(item.get_desktop_file_id()):
+			if util.getItemPath(item.get_desktop_file_id()) is not None:
 				path = util.getUserItemPath()
 				if os.path.isfile(os.path.join(path, item.get_desktop_file_id())):
 					return True
@@ -188,7 +233,7 @@ class MenuEditor:
 				file_id = os.path.split(item.get_desktop_file_path())[1]
 			else:
 				file_id = item.get_menu_id() + '.directory'
-			if util.getDirectoryPath(file_id):
+			if util.getDirectoryPath(file_id) is not None:
 				path = util.getUserDirectoryPath()
 				if os.path.isfile(os.path.join(path, file_id)):
 					return True
@@ -198,10 +243,10 @@ class MenuEditor:
 		dom = self.__getMenu(item).dom
 		if item.get_type() == matemenu.TYPE_ENTRY:
 			self.__addUndo([self.__getMenu(item), item])
-			menu_xml = self.__getXmlMenu(self.__getPath(item.get_parent()), dom, dom)
+			menu_xml = self.__getXmlMenu(self.__getPath(item.get_parent()), dom.documentElement, dom)
 			if visible:
 				self.__addXmlFilename(menu_xml, dom, item.get_desktop_file_id(), 'Include')
-				self.__writeItem(item, no_display=False)
+				self.__writeItem(item, NoDisplay=False)
 			else:
 				self.__addXmlFilename(menu_xml, dom, item.get_desktop_file_id(), 'Exclude')
 			self.__addXmlTextElement(menu_xml, 'AppDir', util.getUserItemPath(), dom)
@@ -210,18 +255,15 @@ class MenuEditor:
 			#don't mess with it if it's empty
 			if len(item.get_contents()) == 0:
 				return
-			menu_xml = self.__getXmlMenu(self.__getPath(item), dom, dom)
+			menu_xml = self.__getXmlMenu(self.__getPath(item), dom.documentElement, dom)
 			for node in self.__getXmlNodesByName(['Deleted', 'NotDeleted'], menu_xml):
 				node.parentNode.removeChild(node)
-			if visible:
-				self.__writeMenu(item, no_display=False)
-			else:
-				self.__writeMenu(item, no_display=True)
+			self.__writeMenu(item, NoDisplay=not visible)
 			self.__addXmlTextElement(menu_xml, 'DirectoryDir', util.getUserDirectoryPath(), dom)
 		self.save()
 
-	def createItem(self, parent, icon, name, comment, command, use_term, before=None, after=None):
-		file_id = self.__writeItem(None, icon, name, comment, command, use_term)
+	def createItem(self, parent, before, after, **kwargs):
+		file_id = self.__writeItem(None, **kwargs)
 		self.insertExternalItem(file_id, parent.menu_id, before, after)
 
 	def insertExternalItem(self, file_id, parent_id, before=None, after=None):
@@ -232,16 +274,12 @@ class MenuEditor:
 		self.__addUndo([self.__getMenu(parent), ('Item', file_id)])
 		self.save()
 
-	def createMenu(self, parent, icon, name, comment, before=None, after=None):
-		file_id = self.__writeMenu(None, icon, name, comment)
-		self.insertExternalMenu(file_id, parent.menu_id, before, after)
-
 	def insertExternalMenu(self, file_id, parent_id, before=None, after=None):
 		menu_id = file_id.rsplit('.', 1)[0]
 		parent = self.__findMenu(parent_id)
 		dom = self.__getMenu(parent).dom
-		self.__addXmlDefaultLayout(self.__getXmlMenu(self.__getPath(parent), dom, dom) , dom)
-		menu_xml = self.__getXmlMenu(self.__getPath(parent) + '/' + menu_id, dom, dom)
+		self.__addXmlDefaultLayout(self.__getXmlMenu(self.__getPath(parent), dom.documentElement, dom) , dom)
+		menu_xml = self.__getXmlMenu(self.__getPath(parent) + [menu_id], dom.documentElement, dom)
 		self.__addXmlTextElement(menu_xml, 'Directory', file_id, dom)
 		self.__positionItem(parent, ('Menu', menu_id), before, after)
 		self.__addUndo([self.__getMenu(parent), ('Menu', file_id)])
@@ -261,10 +299,10 @@ class MenuEditor:
 			parent = item.get_parent()
 		if final:
 			self.__addUndo([self.__getMenu(parent), item])
-		self.__writeItem(item, icon, name, comment, command, use_term)
+		self.__writeItem(item, Icon=icon, Name=name, Comment=comment, Exec=command, Terminal=use_term)
 		if final:
 			dom = self.__getMenu(parent).dom
-			menu_xml = self.__getXmlMenu(self.__getPath(parent), dom, dom)
+			menu_xml = self.__getXmlMenu(self.__getPath(parent), dom.documentElement, dom)
 			self.__addXmlTextElement(menu_xml, 'AppDir', util.getUserItemPath(), dom)
 		self.save()
 
@@ -275,8 +313,8 @@ class MenuEditor:
 		#we don't use this, we just need to make sure the <Menu> exists
 		#otherwise changes won't show up
 		dom = self.__getMenu(menu).dom
-		menu_xml = self.__getXmlMenu(self.__getPath(menu), dom, dom)
-		file_id = self.__writeMenu(menu, icon, name, comment)
+		menu_xml = self.__getXmlMenu(self.__getPath(menu), dom.documentElement, dom)
+		self.__writeMenu(menu, Icon=icon, Name=name, Comment=comment)
 		if final:
 			self.__addXmlTextElement(menu_xml, 'DirectoryDir', util.getUserDirectoryPath(), dom)
 			self.__addUndo([self.__getMenu(menu), menu])
@@ -285,13 +323,19 @@ class MenuEditor:
 	def copyItem(self, item, new_parent, before=None, after=None):
 		dom = self.__getMenu(new_parent).dom
 		file_path = item.get_desktop_file_path()
-		keyfile = util.DesktopParser(file_path)
-		#erase Categories in new file
-		keyfile.set('Categories', ('',))
-		keyfile.set('Hidden', False)
-		file_id = util.getUniqueFileId(item.get_name(), '.desktop')
+		keyfile = GLib.KeyFile()
+		keyfile.load_from_file(file_path, util.KEY_FILE_FLAGS)
+
+		util.fillKeyFile(keyfile, dict(Categories=[], Hidden=False))
+
+		file_id = util.getUniqueFileId(item.get_name().replace(os.sep, '-'), '.desktop')
 		out_path = os.path.join(util.getUserItemPath(), file_id)
-		keyfile.write(open(out_path, 'w'))
+
+		contents, length = keyfile.to_data()
+
+		with open(out_path, 'w') as f:
+			f.write(contents)
+
 		self.__addItem(new_parent, file_id, dom)
 		self.__positionItem(new_parent, ('Item', file_id), before, after)
 		self.__addUndo([self.__getMenu(new_parent), ('Item', file_id)])
@@ -329,32 +373,36 @@ class MenuEditor:
 			return False
 		if menu.get_parent() != new_parent:
 			dom = self.__getMenu(menu).dom
-			root_path = self.__getPath(menu).split('/', 1)[0]
-			xml_root = self.__getXmlMenu(root_path, dom, dom)
-			old_path = self.__getPath(menu).split('/', 1)[1]
-			#root menu's path has no /
-			if '/' in self.__getPath(new_parent):
-				new_path = self.__getPath(new_parent).split('/', 1)[1] + '/' + menu.get_menu_id()
-			else:
-				new_path = menu.get_menu_id()
-			self.__addXmlMove(xml_root, old_path, new_path, dom)
+			path = self.__getPath(menu)
+			root_path = path[0]
+			xml_root = self.__getXmlMenu(root_path, dom.documentElement, dom)
+			old_path = path[1:]
+			new_path = self.__getPath(new_parent)[1:] + [menu.get_menu_id()]
+			self.__addXmlMove(xml_root, '/'.join(old_path), '/'.join(new_path), dom)
 		self.__positionItem(new_parent, menu, before, after)
 		self.__addUndo([self.__getMenu(new_parent),])
 		self.save()
 
 	def moveSeparator(self, separator, new_parent, before=None, after=None):
+		undo = []
+		# remove the original separator if its parent is not the new destination
+		if separator.get_parent() != new_parent:
+			self.deleteSeparator(separator)
+			undo.append(separator)
+		# this adds the new separator to the specified position
 		self.__positionItem(new_parent, separator, before, after)
-		self.__addUndo([self.__getMenu(new_parent),])
+		undo.append(self.__getMenu(new_parent))
+		self.__addUndo(undo)
 		self.save()
 
 	def deleteItem(self, item):
 		self.__addUndo([item,])
-		self.__writeItem(item, hidden=True)
+		self.__writeItem(item, Hidden=True)
 		self.save()
 
 	def deleteMenu(self, menu):
 		dom = self.__getMenu(menu).dom
-		menu_xml = self.__getXmlMenu(self.__getPath(menu), dom, dom)
+		menu_xml = self.__getXmlMenu(self.__getPath(menu), dom.documentElement, dom)
 		self.__addDeleted(menu_xml, dom)
 		self.__addUndo([self.__getMenu(menu),])
 		self.save()
@@ -365,36 +413,11 @@ class MenuEditor:
 		contents.remove(item)
 		layout = self.__createLayout(contents)
 		dom = self.__getMenu(parent).dom
-		menu_xml = self.__getXmlMenu(self.__getPath(parent), dom, dom)
+		menu_xml = self.__getXmlMenu(self.__getPath(parent), dom.documentElement, dom)
 		self.__addXmlLayout(menu_xml, layout, dom)
 		self.__addUndo([self.__getMenu(item.get_parent()),])
 		self.save()
 
-	def revertItem(self, item):
-		if not self.canRevert(item):
-			return
-		self.__addUndo([item,])
-		try:
-			os.remove(item.get_desktop_file_path())
-		except OSError:
-			pass
-		self.save()
-
-	def revertMenu(self, menu):
-		if not self.canRevert(menu):
-			return
-		#wtf happened here? oh well, just bail
-		if not menu.get_desktop_file_path():
-			return
-		self.__addUndo([menu,])
-		file_id = os.path.split(menu.get_desktop_file_path())[1]
-		path = os.path.join(util.getUserDirectoryPath(), file_id)
-		try:
-			os.remove(path)
-		except OSError:
-			pass
-		self.save()
-
 	#private stuff
 	def __addUndo(self, items):
 		self.__undo.append([])
@@ -413,7 +436,7 @@ class MenuEditor:
 				else:
 					continue
 			elif item.get_type() == matemenu.TYPE_DIRECTORY:
-				if item.get_desktop_file_path() == None:
+				if item.get_desktop_file_path() is None:
 					continue
 				file_path = os.path.join(util.getUserDirectoryPath(), os.path.split(item.get_desktop_file_path())[1])
 				if not os.path.isfile(file_path):
@@ -424,9 +447,11 @@ class MenuEditor:
 					file_path = item.get_desktop_file_path()
 			else:
 				continue
-			data = open(file_path).read()
+			with codecs.open(file_path, 'r', 'utf-8') as f:
+				data = f.read()
 			undo_path = util.getUniqueUndoFile(file_path)
-			open(undo_path, 'w').write(data)
+			with codecs.open(undo_path, 'w', 'utf-8') as f:
+				f.write(data)
 			self.__undo[-1].append(undo_path)
 
 	def __getMenu(self, item):
@@ -445,9 +470,9 @@ class MenuEditor:
 		return self.settings
 
 	def __findMenu(self, menu_id, parent=None):
-		if parent == None:
+		if parent is None:
 			menu = self.__findMenu(menu_id, self.applications.tree.root)
-			if menu != None:
+			if menu is not None:
 				return menu
 			else:
 				return self.__findMenu(menu_id, self.settings.tree.root)
@@ -460,7 +485,7 @@ class MenuEditor:
 				if item.menu_id == menu_id:
 					return item
 				menu = self.__findMenu(menu_id, item)
-				if menu != None:
+				if menu is not None:
 					return menu
 
 	def __isVisible(self, item):
@@ -472,52 +497,52 @@ class MenuEditor:
 		elif menu == self.settings:
 			root = self.settings.visible_tree.root
 		if item.get_type() == matemenu.TYPE_DIRECTORY:
-			if self.__findMenu(item.menu_id, root) == None:
+			if self.__findMenu(item.menu_id, root) is None:
 				return False
 		return True
 
-	def __getPath(self, menu, path=None):
-		if not path:
-			path = menu.tree.root.get_menu_id()
-		if menu.get_parent():
-			path = self.__getPath(menu.get_parent(), path)
-			path += '/'
-			path += menu.menu_id
-		return path
+	def __getPath(self, menu):
+		names = []
+		current = menu
+		while current is not None:
+			names.append(current.get_menu_id())
+			current = current.get_parent()
 
-	def __getXmlMenu(self, path, element, dom):
-		if '/' in path:
-			(name, path) = path.split('/', 1)
-		else:
-			name = path
-			path = ''
+		# XXX - don't append root menu name, mozo doesn't
+		# expect it. look into this more.
+		names.pop(-1)
+		return names[::-1]
 
-		found = None
+	def __getXmlMenuPart(self, element, name):
 		for node in self.__getXmlNodesByName('Menu', element):
 			for child in self.__getXmlNodesByName('Name', node):
 				if child.childNodes[0].nodeValue == name:
-					if path:
-						found = self.__getXmlMenu(path, node, dom)
-					else:
-						found = node
-					break
-			if found:
-				break
-		if not found:
-			node = self.__addXmlMenuElement(element, name, dom)
-			if path:
-				found = self.__getXmlMenu(path, node, dom)
-			else:
-				found = node
+					return node
+		return None
 
-		return found
+	def __getXmlMenu(self, path, element, dom):
+		for name in path:
+			found = self.__getXmlMenuPart(element, name)
+			if found is not None:
+				element = found
+			else:
+				element = self.__addXmlMenuElement(element, name, dom)
+		return element
 
 	def __addXmlMenuElement(self, element, name, dom):
+		if isinstance(name, bytes):
+			name = name.decode('utf-8')
+
 		node = dom.createElement('Menu')
 		self.__addXmlTextElement(node, 'Name', name, dom)
 		return element.appendChild(node)
 
 	def __addXmlTextElement(self, element, name, text, dom):
+		if isinstance(name, bytes):
+			name = name.decode('utf-8')
+		if isinstance(text, bytes):
+			text = text.decode('utf-8')
+
 		for temp in element.childNodes:
 			if temp.nodeName == name:
 				if temp.childNodes[0].nodeValue == text:
@@ -528,6 +553,9 @@ class MenuEditor:
 		return element.appendChild(node)
 
 	def __addXmlFilename(self, element, dom, filename, type = 'Include'):
+		if isinstance(filename, bytes):
+			filename = filename.decode('utf-8')
+
 		# remove old filenames
 		for node in self.__getXmlNodesByName(['Include', 'Exclude'], element):
 			if node.childNodes[0].nodeName == 'Filename' and node.childNodes[0].childNodes[0].nodeValue == filename:
@@ -542,65 +570,59 @@ class MenuEditor:
 		node = dom.createElement('Deleted')
 		return element.appendChild(node)
 
-	def __writeItem(self, item=None, icon=None, name=None, comment=None, command=None, use_term=None, no_display=None, startup_notify=None, hidden=None):
-		if item:
+	def __makeKeyFile(self, file_path, kwargs):
+		if 'KeyFile' in kwargs:
+			return kwargs['KeyFile']
+
+		keyfile = GLib.KeyFile()
+
+		if file_path is not None:
+			keyfile.load_from_file(file_path, util.KEY_FILE_FLAGS)
+
+		util.fillKeyFile(keyfile, kwargs)
+		return keyfile
+
+	def __writeItem(self, item, **kwargs):
+		if item is not None:
 			file_path = item.get_desktop_file_path()
-			file_id = item.get_desktop_file_id()
-			keyfile = util.DesktopParser(file_path)
-		elif item == None and name == None:
-			raise Exception('New menu items need a name')
 		else:
-			file_id = util.getUniqueFileId(item.get_name().replace(os.sep, '-'), '.desktop')
-			keyfile = util.DesktopParser()
-		if icon:
-			keyfile.set('Icon', icon)
-			keyfile.set('Icon', icon, self.locale)
-		if name:
-			keyfile.set('Name', name)
-			keyfile.set('Name', name, self.locale)
-		if comment:
-			keyfile.set('Comment', comment)
-			keyfile.set('Comment', comment, self.locale)
-		if command:
-			keyfile.set('Exec', command)
-		if use_term != None:
-			keyfile.set('Terminal', use_term)
-		if no_display != None:
-			keyfile.set('NoDisplay', no_display)
-		if startup_notify != None:
-			keyfile.set('StartupNotify', startup_notify)
-		if hidden != None:
-			keyfile.set('Hidden', hidden)
-		out_path = os.path.join(util.getUserItemPath(), file_id)
-		keyfile.write(open(out_path, 'w'))
+			file_path = None
+
+		keyfile = self.__makeKeyFile(file_path, kwargs)
+
+		if item is not None:
+			file_id = item.get_desktop_file_id()
+		else:
+			file_id = util.getUniqueFileId(keyfile.get_string(GLib.KEY_FILE_DESKTOP_GROUP, 'Name'), '.desktop')
+
+		contents, length = keyfile.to_data()
+
+		with open(os.path.join(util.getUserItemPath(), file_id), 'w') as f:
+		        f.write(contents)
 		return file_id
 
-	def __writeMenu(self, menu=None, icon=None, name=None, comment=None, no_display=None):
-		if menu:
+	def __writeMenu(self, menu, **kwargs):
+		if menu is not None:
 			file_id = os.path.split(menu.get_desktop_file_path())[1]
 			file_path = menu.get_desktop_file_path()
-			keyfile = util.DesktopParser(file_path)
-		elif menu == None and name == None:
+			keyfile = GLib.KeyFile()
+			keyfile.load_from_file(file_path, util.KEY_FILE_FLAGS)
+		elif menu is None and 'Name' not in kwargs:
 			raise Exception('New menus need a name')
 		else:
-			file_id = util.getUniqueFileId(name, '.directory')
-			keyfile = util.DesktopParser(file_type='Directory')
-		if icon:
-			keyfile.set('Icon', icon)
-		if name:
-			keyfile.set('Name', name)
-			keyfile.set('Name', name, self.locale)
-		if comment:
-			keyfile.set('Comment', comment)
-			keyfile.set('Comment', comment, self.locale)
-		if no_display != None:
-			keyfile.set('NoDisplay', no_display)
-		out_path = os.path.join(util.getUserDirectoryPath(), file_id)
-		keyfile.write(open(out_path, 'w'))
+			file_id = util.getUniqueFileId(kwargs['Name'], '.directory')
+			keyfile = GLib.KeyFile()
+
+		util.fillKeyFile(keyfile, kwargs)
+
+		contents, length = keyfile.to_data()
+
+		with open(os.path.join(util.getUserDirectoryPath(), file_id), 'w') as f:
+			f.write(contents)
 		return file_id
 
 	def __getXmlNodesByName(self, name, element):
-		for	child in element.childNodes:
+		for child in element.childNodes:
 			if child.nodeType == xml.dom.Node.ELEMENT_NODE:
 				if isinstance(name, str) and child.nodeName == name:
 					yield child
@@ -608,18 +630,6 @@ class MenuEditor:
 					if child.nodeName in name:
 						yield child
 
-	def __remove_whilespace_nodes(self, node):
-		remove_list = []
-		for child in node.childNodes:
-			if child.nodeType == xml.dom.minidom.Node.TEXT_NODE:
-				child.data = child.data.strip()
-				if not child.data.strip():
-					remove_list.append(child)
-			elif child.hasChildNodes():
-				self.__remove_whilespace_nodes(child)
-		for node in remove_list:
-			node.parentNode.removeChild(node)
-
 	def __addXmlMove(self, element, old, new, dom):
 		if not self.__undoMoves(element, old, new, dom):
 			node = dom.createElement('Move')
@@ -682,30 +692,29 @@ class MenuEditor:
 		return layout
 
 	def __addItem(self, parent, file_id, dom):
-		xml_parent = self.__getXmlMenu(self.__getPath(parent), dom, dom)
+		xml_parent = self.__getXmlMenu(self.__getPath(parent), dom.documentElement, dom)
 		self.__addXmlFilename(xml_parent, dom, file_id, 'Include')
 
-	def __deleteItem(self, parent, file_id, dom, before=None, after=None):
-		xml_parent = self.__getXmlMenu(self.__getPath(parent), dom, dom)
-		self.__addXmlFilename(xml_parent, dom, file_id, 'Exclude')
-
 	def __positionItem(self, parent, item, before=None, after=None):
-		if not before and not after:
-			return
 		if after:
 			index = parent.contents.index(after) + 1
 		elif before:
 			index = parent.contents.index(before)
+		else:
+			# append the item to the list
+			index = len(parent.contents)
 		contents = parent.contents
 		#if this is a move to a new parent you can't remove the item
-		try:
+		if item in contents:
+			# decrease the destination index, if we shorten the list
+			if (before and (contents.index(item) < index)) \
+			    or (after and (contents.index(item) < index - 1)):
+				index -= 1
 			contents.remove(item)
-		except:
-			pass
 		contents.insert(index, item)
 		layout = self.__createLayout(contents)
 		dom = self.__getMenu(parent).dom
-		menu_xml = self.__getXmlMenu(self.__getPath(parent), dom, dom)
+		menu_xml = self.__getXmlMenu(self.__getPath(parent), dom.documentElement, dom)
 		self.__addXmlLayout(menu_xml, layout, dom)
 
 	def __undoMoves(self, element, old, new, dom):
@@ -740,7 +749,7 @@ class MenuEditor:
 						if name == os.path.split(new)[1]:
 							#copy app and dir directory info from old <Menu>
 							root_path = dom.getElementsByTagName('Menu')[0].getElementsByTagName('Name')[0].childNodes[0].nodeValue
-							xml_menu = self.__getXmlMenu(root_path + '/' + new, dom, dom)
+							xml_menu = self.__getXmlMenu(root_path + '/' + new, dom.documentElement, dom)
 							for app_dir in node.getElementsByTagName('AppDir'):
 								xml_menu.appendChild(app_dir)
 							for dir_dir in node.getElementsByTagName('DirectoryDir'):
